<?php

namespace Drupal\acquia_contenthub\EventSubscriber\UnserializeContentField;

use Drupal\acquia_contenthub\Event\UnserializeCdfEntityFieldEvent;
use Drupal\Core\Entity\EntityInterface;
use Psr\Log\LoggerInterface;

/**
 * Base class for unserializers that handle referenced entities.
 */
class FieldEntityReferenceBase {

  /**
   * Content hub logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * FieldEntityReferenceBase constructor.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   Content hub logger.
   */
  public function __construct(LoggerInterface $logger) {
    $this->logger = $logger;
  }

  /**
   * Get an entity from the dependency stack.
   *
   * @param string $uuid
   *   The uuid of the entity to retrieve.
   * @param \Drupal\acquia_contenthub\Event\UnserializeCdfEntityFieldEvent $event
   *   The subscribed event.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The retrieved entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Exception
   */
  protected function getEntity(string $uuid, UnserializeCdfEntityFieldEvent $event): ?EntityInterface {
    // Check stack first because uuids are referenced from origin, not local.
    if ($event->getStack()->hasDependency($uuid)) {
      return $event->getStack()->getDependency($uuid)->getEntity();
    }
    // Only fall back to local uuids as an absolute last resort.
    if (empty($event->getFieldMetadata()['target'])) {
      throw new \Exception(sprintf("The %s field does not specify a metadata target. This is likely due to an unresolved dependency export process. Please check your relationships.", $event->getFieldName()));
    }
    $storage = \Drupal::entityTypeManager()->getStorage($event->getFieldMetadata()['target']);
    $entities = $storage->loadByProperties(['uuid' => $uuid]);
    return !empty($entities) ? reset($entities) : NULL;
  }

  /**
   * Log message if entity is missing and is not pruned.
   *
   * @param string $uuid
   *   Entity uuid.
   */
  protected function log(string $uuid): void {
    $this->logger->error(
      'Entity with {uuid} not found in DependencyStack or subscriber\'s database while unserializing field values.',
      [
        'uuid' => $uuid,
      ]
    );
  }

}
